@@ -1,11 +1,11 @@
 #
-# $Id: Splunklib.pm,v e46bf13852d9 2015/10/03 10:12:28 gomor $
+# $Id: Splunklib.pm,v 927d5bf7d37e 2015/10/03 12:57:38 gomor $
 #
 package Splunklib;
 use strict;
 use warnings;
 
-our $VERSION = '0.22';
+our $VERSION = '0.23';
 
 1;
 
@@ -23,7 +23,14 @@ Splunklib - the Splunk SDK to create custom commands in Perl
    use FindBin qw($Bin);
    use lib "$Bin/lib";
 
-   use Splunklib::Intersplunk qw(readResults outputResults);
+   use Splunklib::Intersplunk qw(readResults outputResults isGetInfo outputGetInfo);
+
+   # GetInfo support
+   # @ARGV example: "__GETINFO__ field=_raw action=encode"
+   if (isGetInfo(\@ARGV)) {
+      outputGetInfo(undef, \*STDOUT);
+      exit(0);
+   }
 
    # @ARGV example: "field=_raw action=encode"
    my $field = '_raw';    # Default to encode/decode _raw
